#include <InterfaceDefs.h>

/*============================================================================*/
/*
		コンテナローダー　単純リスト／準単純リスト		for Light MIDI
*/
const char*	gAddOnName = "Container Loader Raw List";
const char*	gAddOnVer = "Ver 1.10";
const char*	gCright = "Copyright "B_UTF8_COPYRIGHT" 2002 by SHINTA";
const char*	gDescription = "Tells MIDI files in the raw list / semi raw list.\n"
			"Raw List: Only file names are written.\n"
			"Semi Raw List: File names and other informations ( header etc. ) are written.";
/*============================================================================*/

/*==============================================================================
 Ver. |					主な改良点
--------------------------------------------------------------------------------
 ---- | 2002.01.02.We 開発着手。
 1.00 | 2002.01.02.We オリジナルバージョン。
 1.10 | 2002.01.09.Tu 第 2 世代仕様に準拠した。
==============================================================================*/

/*=== Memo =====================================================================
==============================================================================*/

//------------------------------------------------------------------------------
#include "CLRaw.h"
//------------------------------------------------------------------------------
// BeOS
#include <File.h>
// C++
// Add
// Proj
//------------------------------------------------------------------------------
//==============================================================================
TCLRaw::TCLRaw(const string& oPath)
		: SUPER(oPath)
{
}
//------------------------------------------------------------------------------
TCLRaw::~TCLRaw()
{
}
//------------------------------------------------------------------------------
string	TCLRaw::Copyright() const
{
	return gCright;
}
//------------------------------------------------------------------------------
string	TCLRaw::Description() const
{
	return gDescription;
}
//------------------------------------------------------------------------------
/*	''''''''''''''''''''''''''''''
1 行読み込む
Read 1 line
,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,	*/
ssize_t	TCLRaw::GetLine(BFile* oFile,char* oBuf,int oBufSize) const
{
	ssize_t	aCount = 0;

	while ( aCount < oBufSize-1 ) {
		if ( oFile->Read(&oBuf[aCount],1) < 1 )
			break;
		if ( oBuf[aCount++] == '\n' ) {
			oBuf[aCount-1] = '\0';
			if ( aCount > 1 && oBuf[aCount-2] == '\r' ) {
				oBuf[aCount-2] = '\0';
			}
			break;
		}
	}
	return aCount;
}
//------------------------------------------------------------------------------
bool	TCLRaw::IsLoadable(const entry_ref& oERef,TSupportFmt* oFmt) const
{
	char		aBuf[FREAD_BUF_SIZE];
	BFile		aFile(&oERef,B_READ_ONLY);	// フォルダを食っても InitCheck() は B_OK になってしまう

	if ( BEntry(&oERef).IsDirectory() )
		return false;
	// ファイルの 1 行目が MIDI ファイルだったら単純リスト（ファイル名の羅列）と判定する
	// If the first line is a MIDI file name, oERef is raw list
	GetLine(&aFile,aBuf,FREAD_BUF_SIZE);
	if ( IsMidiFileName(aBuf) ) {
		*oFmt = SF_RAW_LIST;
		return true;
	}
	// 後続の数行のうち、どれか 1 つ以上が MIDI ファイルだったら準単純リスト（ヘッダなどがある）と判定する
	// If there is a MIDI file name, oERef is semi raw list
	for ( int i = 0 ; i < SEMI_RAW_FMT_DISTINCT_BREAK_LINES ; i++ ) {
		GetLine(&aFile,aBuf,FREAD_BUF_SIZE);
		if ( IsMidiFileName(aBuf) ) {
			*oFmt = SF_SEMI_RAW_LIST;
			return true;
		}
	}
	return false;
}
//------------------------------------------------------------------------------
/*	''''''''''''''''''''''''''''''
MIDI ファイル名（拡張子が .mid ）かどうかを判定
Whether oFileName is a MIDI file name or not
,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,	*/
bool	TCLRaw::IsMidiFileName(const char* oFileName) const
{
	int		aFileNameLen = strlen(oFileName);
	int		aExtMidiLen = strlen(FILE_EXT_MIDI);

	if ( aFileNameLen < aExtMidiLen )
		return false;
	return (strcasecmp(&oFileName[aFileNameLen-aExtMidiLen],FILE_EXT_MIDI) == 0);
}
//------------------------------------------------------------------------------
status_t	TCLRaw::Load(const entry_ref& oERef,list<string>* oFiles)
{
	char			aBuf[FREAD_BUF_SIZE];
	BFile			aFile;

	if ( aFile.SetTo(&oERef,B_READ_ONLY) != B_OK )
		return B_ERROR;
	// MIDI ファイル名が書かれていたら oFiles にそのファイル名を追加していく
	// Add MIDI files to oFiles
	while ( GetLine(&aFile,aBuf,FREAD_BUF_SIZE) > 0 ) {
		if ( IsMidiFileName(aBuf) ) {
			oFiles->push_back(aBuf);
		}
	}
	return B_OK;
}
//------------------------------------------------------------------------------
string	TCLRaw::Name() const
{
	return gAddOnName;
}
//------------------------------------------------------------------------------
string	TCLRaw::Version() const
{
	return gAddOnVer;
}
//------------------------------------------------------------------------------
//==============================================================================
TContainerLoader*	make_container_loader(const string& oPath,image_id oImgID)
{
	return new TCLRaw(oPath);
}
//==============================================================================
